package com.fjhx.email.controller;

import cn.hutool.core.io.IORuntimeException;
import cn.hutool.core.thread.ThreadUtil;
import com.fjhx.email.config.base.R;
import com.fjhx.email.config.exception.ServiceException;
import com.fjhx.email.entity.EnterpriseMailbox;
import com.fjhx.email.entity.PersonalMailbox;
import com.fjhx.email.entity.dto.*;
import com.fjhx.email.entity.vo.MessageDetailVo;
import com.fjhx.email.entity.vo.MessageVo;
import com.fjhx.email.service.IEnterpriseMailboxService;
import com.fjhx.email.service.IMailService;
import com.fjhx.email.service.IPersonalMailboxService;
import com.fjhx.email.utils.HxHttpUtil;
import com.fjhx.email.utils.PageWrapper;
import com.fjhx.email.utils.email.SmtpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;

import javax.mail.MessagingException;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/mailService")
public class MailController {

    @Autowired
    private IMailService mailService;

    @Autowired
    private IEnterpriseMailboxService enterpriseMailboxService;

    @Autowired
    private IPersonalMailboxService personalMailboxService;

    /**
     * 每小时同步一次在线用户
     */
    @Scheduled(cron = "0 0 0/1 * * ?")
    public void checkStart() {
        try {
            synchronized (this) {
                List<Long> onlineUserIdList = HxHttpUtil.getOnlineUserIdList();
                MailSyncInfo.mailboxInfoList = mailService.getMailboxInfoListByUserId(onlineUserIdList, MailSyncInfo.mailType);
            }
        } catch (IORuntimeException e) {
            log.error("获取主服务在线用户超时");
            ThreadUtil.sleep(1000 * 10);
            checkStart();
        } catch (Exception e) {
            log.error("其他未知错误", e);
            ThreadUtil.sleep(1000 * 10);
            checkStart();
        }
    }

    /**
     * 统计登录用户id
     */
    @GetMapping("/userLogin/{userId}")
    public R userLogin(@PathVariable("userId") Long userId) {
        try {
            synchronized (this) {

                List<MailboxInfo> mailboxInfoList = mailService.getMailboxInfoListByUserId(Collections.singletonList(userId), MailSyncInfo.mailType);

                if (mailboxInfoList.size() == 0) {
                    return R.ok();
                }

                List<Long> mailBoxInfoIdList = MailSyncInfo.mailboxInfoList.stream()
                        .map(MailboxInfo::getId).collect(Collectors.toList());

                for (MailboxInfo mailboxInfo : mailboxInfoList) {
                    if (!mailBoxInfoIdList.contains(mailboxInfo.getId())) {
                        MailSyncInfo.mailboxInfoList.add(mailboxInfo);
                    }
                }

            }
        } catch (Exception e) {
            log.error("登录信息异常", e);
            return userLogin(userId);
        }
        return R.ok();
    }

    /**
     * 获取用户邮箱列表
     */
    @GetMapping("/getUserEmailList/{userId}")
    public R getUserEmailList(@PathVariable("userId") Long userId) {
        List<MailboxInfo> mailboxInfoList = mailService.getMailboxInfoListByUserId(Collections.singletonList(userId), null);

        // 赋值文件夹未读邮件数量
        mailService.setUnreadMessageCount(mailboxInfoList);

        return R.ok(mailboxInfoList);
    }

    /**
     * 获取用户邮箱列表
     */
    @PostMapping("/getMessagePage")
    public R getMessagePage(@RequestBody GetMessagePageDto dto) {
        PageWrapper<MessageVo> messagePage = mailService.getMessagePage(dto);
        return R.ok(messagePage);
    }

    /**
     * 获取邮件详情
     */
    @PostMapping("/getMessageDetail")
    public R getMessageDetail(@RequestBody GetMessagePageDto dto) {
        MessageDetailVo vo = mailService.getMessageDetail(dto);
        return R.ok(vo);
    }

    /**
     * 发送邮件
     */
    @PostMapping("/sendMail/{host}/{user}/{password}")
    public R sendMail(@PathVariable("host") String host, @PathVariable("user") String user,
                      @PathVariable("password") String password, @RequestBody SendDto sendDto) {
        try {
            SmtpUtil.sendMail(host, user, password, sendDto);
        } catch (Exception e) {
            log.error("发送邮件失败：", e);
            throw new ServiceException("发送邮件失败：{}", e.getMessage());
        }
        return R.ok();
    }

    /**
     * 添加企业邮箱
     */
    @PostMapping("/addEnterpriseMailbox")
    public R addEnterpriseMailbox(@RequestBody EnterpriseMailbox enterpriseMailbox) {
        enterpriseMailboxService.add(enterpriseMailbox);
        userLogin(enterpriseMailbox.getUserId());
        return R.ok();
    }

    /**
     * 添加个人邮箱
     */
    @PostMapping("/addPersonalMailbox")
    public R addPersonalMailbox(@RequestBody PersonalMailbox personalMailbox) {
        personalMailboxService.add(personalMailbox);
        userLogin(personalMailbox.getUserId());
        return R.ok();
    }

    /**
     * 设置已读
     */
    @PostMapping("/setSeen/{host}/{user}/{password}")
    public R setSeen(@PathVariable("host") String host, @PathVariable("user") String user,
                     @PathVariable("password") String password, @RequestBody SetSeenDto dto) {
        try {
            mailService.setSeen(host, user, password, dto);
        } catch (MessagingException e) {
            log.error("设置已读失败：", e);
            throw new ServiceException("设置已读失败：{}", e.getMessage());
        }
        return R.ok();
    }

}
